#include "panell.h"
#include "ui_panell.h"
#include <QDebug>
#include <QMessageBox>

Panell::Panell(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::Panell)
{
    ui->setupUi(this);

    m_client = new QMqttClient(this);
    m_client->setHostname(ui->lineEditUrlBroker->text());
    m_client->setPort(ui->lineEditBrokerPort->text().toInt());

    connect(m_client, &QMqttClient::stateChanged, this, &Panell::updateLogStateChange);
    connect(m_client, &QMqttClient::disconnected, this, &Panell::brokerDisconnected);
    connect(m_client, &QMqttClient::messageReceived, this, [this](const QByteArray &message, const QMqttTopicName &topic) {
        const QString content = QDateTime::currentDateTime().toString()
                    + QLatin1String(" Received Topic: ")
                    + topic.name()
                    + QLatin1String(" Message: ")
                    + message
                    + QLatin1Char('\n');
        vGestionaResposta(topic.name(),message);
        ui->plainTextEdit->insertPlainText(content);
    });
    vEnableTopLayout(false);
}

Panell::~Panell()
{
    delete ui;
}

void Panell::on_btConnect_clicked()
{
    if (m_client->state() == QMqttClient::Disconnected) {
        ui->lineEditUrlBroker->setEnabled(false);
        ui->lineEditBrokerPort->setEnabled(false);
        ui->btConnect->setText("&Disconnect");
        m_client->connectToHost();
        vEnableTopLayout(true);
    } else {
        ui->lineEditUrlBroker->setEnabled(true);
        ui->lineEditBrokerPort->setEnabled(true);
        ui->btConnect->setText("&Connect");
        m_client->disconnectFromHost();
        vEnableTopLayout(false);
    }
}

void Panell::updateLogStateChange()
{
    int nEstat = -1;
    const QString content = QDateTime::currentDateTime().toString()
                    + QLatin1String(": State Change")
                    + QString::number(nEstat = m_client->state())
                    + QLatin1Char('\n');
    ui->plainTextEdit->insertPlainText(content);
    qDebug() << content;
    if(nEstat == 2){
        subscription(TEMA_T1);
        subscription(TEMA_CABAL);
    }
}

void Panell::brokerDisconnected()
{
    ui->lineEditUrlBroker->setEnabled(true);
    ui->lineEditBrokerPort->setEnabled(true);
    ui->btConnect->setText(tr("&Connect"));
}

void Panell::subscription(QString topic)
{
    auto subscription = m_client->subscribe(topic);
    if (!subscription) {
        QMessageBox::critical(this, QLatin1String("Error"), QLatin1String("Could not subscribe. Is there a valid connection?"));
        return;
    }else
        qDebug() << "Subscrit a " << topic;
}

void Panell::vGestionaResposta(QString tema,QString missatge){
    if(tema == TEMA_T1){
        ui->progressBarT1->setValue(missatge.toInt());
    }
    if(tema == TEMA_CABAL){
        ui->etCabal->setText(missatge);
    }
}

void Panell::vEnableTopLayout(bool b){
    ui->verticalSliderT3->setEnabled(b);
    ui->etCabal->setEnabled(b);
    ui->progressBarT1->setEnabled(b);
}

//#define TEMA_T3 "/dam/mesura/T3"
//#define TEMA_CABAL "/dam/mesura/cabal"
void Panell::on_verticalSliderT3_valueChanged(int value)
{
    if (m_client->state() == QMqttClient::Connected) {
        publica(TEMA_T3,QString::number(value));
    }
}

void Panell::publica(QString tema,QString missatge)
{
    if (m_client->publish(tema, missatge.toUtf8()) == -1)
        QMessageBox::critical(this, QLatin1String("Error"), QLatin1String("Could not publish message"));
}


